@@ -42,8 +42,10 @@ from gazoo_device.utility import usb_utils
 logger = gdm_logger.get_logger()
 
 JLINK_COMMS_PRODUCT_NAME = "J-Link"
-SEGGER_COMMS_ADDRESS = "SEGGER_J-Link"
-SILAB_COMMS_ADDRESS = "Silicon_Labs_J-Link"
+CP2104_COMMS_PRODUCT_NAME = "CP2104"
+NRF_DK_COMMS_ADDRESS = "SEGGER_J-Link"
+EFR32_COMMS_ADDRESS = "Silicon_Labs_J-Link"
+ESP32_M5STACK_COMMS_ADDRESS = "Silicon_Labs_CP2104"
 
 
 def get_specific_serial_addresses(match_criteria):
@@ -403,13 +405,16 @@ class PigweedSerialComms(CommunicationType):
 
   @classmethod
   def get_comms_addresses(cls):
-    include_address = [SEGGER_COMMS_ADDRESS, SILAB_COMMS_ADDRESS]
+    include_product = [JLINK_COMMS_PRODUCT_NAME, CP2104_COMMS_PRODUCT_NAME]
+    include_address = [NRF_DK_COMMS_ADDRESS,
+                       EFR32_COMMS_ADDRESS,
+                       ESP32_M5STACK_COMMS_ADDRESS]
     match_criteria = {
         "product_name": {
-            "include_regex": JLINK_COMMS_PRODUCT_NAME
+            "include_regex": "|".join(include_product)
         },
         "address": {
-            # TODO(gdm-authors): Add address regex for Mac.
+            # TODO(b/187163734): Add address regex for Mac.
             "include_regex": "|".join(include_address)
         }
     }
